package org.eclipse.jdt.internal.compiler.apt.dispatch;

package .tmp;

import javax.lang.model.element.AnnotationMirror;
import javax.lang.model.element.AnnotationValue;
import javax.lang.model.element.Element;
import javax.lang.model.element.ElementKind;
import javax.tools.Diagnostic;
import org.eclipse.jdt.internal.compiler.CompilationResult;
import org.eclipse.jdt.internal.compiler.apt.dispatch.AptProblem;
import org.eclipse.jdt.internal.compiler.apt.model.AnnotationMemberValue;
import org.eclipse.jdt.internal.compiler.apt.model.AnnotationMirrorImpl;
import org.eclipse.jdt.internal.compiler.apt.model.ExecutableElementImpl;
import org.eclipse.jdt.internal.compiler.apt.model.TypeElementImpl;
import org.eclipse.jdt.internal.compiler.apt.model.VariableElementImpl;
import org.eclipse.jdt.internal.compiler.ast.AbstractMethodDeclaration;
import org.eclipse.jdt.internal.compiler.ast.Annotation;
import org.eclipse.jdt.internal.compiler.ast.ArrayInitializer;
import org.eclipse.jdt.internal.compiler.ast.Expression;
import org.eclipse.jdt.internal.compiler.ast.FieldDeclaration;
import org.eclipse.jdt.internal.compiler.ast.LocalDeclaration;
import org.eclipse.jdt.internal.compiler.ast.MemberValuePair;
import org.eclipse.jdt.internal.compiler.ast.TypeDeclaration;
import org.eclipse.jdt.internal.compiler.impl.ReferenceContext;
import org.eclipse.jdt.internal.compiler.lookup.AnnotationBinding;
import org.eclipse.jdt.internal.compiler.lookup.AptSourceLocalVariableBinding;
import org.eclipse.jdt.internal.compiler.lookup.Binding;
import org.eclipse.jdt.internal.compiler.lookup.FieldBinding;
import org.eclipse.jdt.internal.compiler.lookup.MethodBinding;
import org.eclipse.jdt.internal.compiler.lookup.ReferenceBinding;
import org.eclipse.jdt.internal.compiler.lookup.SourceTypeBinding;
import org.eclipse.jdt.internal.compiler.util.Util;

public class BaseMessagerImpl {
  static final String[] NO_ARGUMENTS = new String[0];
  
  public static AptProblem createProblem(Diagnostic.Kind kind, CharSequence msg, Element e, AnnotationMirror a, AnnotationValue v) {
    AbstractMethodDeclaration abstractMethodDeclaration;
    int severity;
    ReferenceContext referenceContext = null;
    Annotation[] elementAnnotations = null;
    int startPosition = 0;
    int endPosition = 0;
    if (e != null) {
      TypeElementImpl typeElementImpl;
      Binding typeBinding;
      ExecutableElementImpl executableElementImpl;
      Binding binding;
      VariableElementImpl variableElementImpl;
      switch (e.getKind()) {
        case ENUM:
        case CLASS:
        case null:
        case INTERFACE:
          typeElementImpl = (TypeElementImpl)e;
          typeBinding = typeElementImpl._binding;
          if (typeBinding instanceof SourceTypeBinding) {
            SourceTypeBinding sourceTypeBinding = (SourceTypeBinding)typeBinding;
            TypeDeclaration typeDeclaration = (TypeDeclaration)sourceTypeBinding.scope.referenceContext();
            TypeDeclaration typeDeclaration1 = typeDeclaration;
            elementAnnotations = typeDeclaration.annotations;
            startPosition = typeDeclaration.sourceStart;
            endPosition = typeDeclaration.sourceEnd;
          } 
          break;
        case METHOD:
        case CONSTRUCTOR:
          executableElementImpl = (ExecutableElementImpl)e;
          binding = executableElementImpl._binding;
          if (binding instanceof MethodBinding) {
            MethodBinding methodBinding = (MethodBinding)binding;
            AbstractMethodDeclaration sourceMethod = methodBinding.sourceMethod();
            if (sourceMethod != null) {
              abstractMethodDeclaration = sourceMethod;
              elementAnnotations = sourceMethod.annotations;
              startPosition = sourceMethod.sourceStart;
              endPosition = sourceMethod.sourceEnd;
            } 
          } 
          break;
        case FIELD:
        case PARAMETER:
          variableElementImpl = (VariableElementImpl)e;
          binding = variableElementImpl._binding;
          if (binding instanceof FieldBinding) {
            FieldBinding fieldBinding = (FieldBinding)binding;
            FieldDeclaration fieldDeclaration = fieldBinding.sourceField();
            if (fieldDeclaration != null) {
              ReferenceBinding declaringClass = fieldBinding.declaringClass;
              if (declaringClass instanceof SourceTypeBinding) {
                SourceTypeBinding sourceTypeBinding = (SourceTypeBinding)declaringClass;
                TypeDeclaration typeDeclaration = (TypeDeclaration)sourceTypeBinding.scope.referenceContext();
                TypeDeclaration typeDeclaration1 = typeDeclaration;
              } 
              elementAnnotations = fieldDeclaration.annotations;
              startPosition = fieldDeclaration.sourceStart;
              endPosition = fieldDeclaration.sourceEnd;
            } 
            break;
          } 
          if (binding instanceof AptSourceLocalVariableBinding) {
            AptSourceLocalVariableBinding parameterBinding = (AptSourceLocalVariableBinding)binding;
            LocalDeclaration parameterDeclaration = parameterBinding.declaration;
            if (parameterDeclaration != null) {
              MethodBinding methodBinding = parameterBinding.methodBinding;
              if (methodBinding != null)
                abstractMethodDeclaration = methodBinding.sourceMethod(); 
              elementAnnotations = parameterDeclaration.annotations;
              startPosition = parameterDeclaration.sourceStart;
              endPosition = parameterDeclaration.sourceEnd;
            } 
          } 
          break;
      } 
    } 
    StringBuilder builder = new StringBuilder();
    if (msg != null)
      builder.append(msg); 
    if (a != null && elementAnnotations != null) {
      AnnotationBinding annotationBinding = ((AnnotationMirrorImpl)a)._binding;
      Annotation annotation = findAnnotation(elementAnnotations, annotationBinding);
      if (annotation != null) {
        startPosition = annotation.sourceStart;
        endPosition = annotation.sourceEnd;
        if (v != null && v instanceof AnnotationMemberValue) {
          MethodBinding methodBinding = ((AnnotationMemberValue)v).getMethodBinding();
          MemberValuePair[] memberValuePairs = annotation.memberValuePairs();
          MemberValuePair memberValuePair = null;
          for (int i = 0; memberValuePair == null && i < memberValuePairs.length; i++) {
            if (methodBinding == (memberValuePairs[i]).binding)
              memberValuePair = memberValuePairs[i]; 
          } 
          if (memberValuePair != null) {
            startPosition = memberValuePair.sourceStart;
            endPosition = memberValuePair.sourceEnd;
          } 
        } 
      } 
    } 
    int lineNumber = 0;
    int columnNumber = 1;
    char[] fileName = null;
    if (abstractMethodDeclaration != null) {
      CompilationResult result = abstractMethodDeclaration.compilationResult();
      fileName = result.fileName;
      int[] lineEnds = null;
      lineNumber = (startPosition >= 0) ? 
        Util.getLineNumber(startPosition, lineEnds = result.getLineSeparatorPositions(), 0, lineEnds.length - 1) : 
        0;
      columnNumber = (startPosition >= 0) ? 
        Util.searchColumnNumber(result.getLineSeparatorPositions(), lineNumber, startPosition) : 
        0;
    } 
    switch (kind) {
      case null:
        severity = 1;
        break;
      default:
        severity = 0;
        break;
    } 
    return new AptProblem(
        (ReferenceContext)abstractMethodDeclaration, 
        fileName, 
        String.valueOf(builder), 
        0, 
        NO_ARGUMENTS, 
        severity, 
        startPosition, 
        endPosition, 
        lineNumber, 
        columnNumber);
  }
  
  private static Annotation findAnnotation(Annotation[] elementAnnotations, AnnotationBinding annotationBinding) {
    for (int i = 0; i < elementAnnotations.length; i++) {
      Annotation annotation = findAnnotation(elementAnnotations[i], annotationBinding);
      if (annotation != null)
        return annotation; 
    } 
    return null;
  }
  
  private static Annotation findAnnotation(Annotation elementAnnotation, AnnotationBinding annotationBinding) {
    if (annotationBinding == elementAnnotation.getCompilerAnnotation())
      return elementAnnotation; 
    MemberValuePair[] memberValuePairs = elementAnnotation.memberValuePairs();
    byte b;
    int i;
    MemberValuePair[] arrayOfMemberValuePair1;
    for (i = (arrayOfMemberValuePair1 = memberValuePairs).length, b = 0; b < i; ) {
      MemberValuePair mvp = arrayOfMemberValuePair1[b];
      Expression v = mvp.value;
      if (v instanceof Annotation) {
        Annotation a = findAnnotation((Annotation)v, annotationBinding);
        if (a != null)
          return a; 
      } else if (v instanceof ArrayInitializer) {
        Expression[] expressions = ((ArrayInitializer)v).expressions;
        byte b1;
        int j;
        Expression[] arrayOfExpression1;
        for (j = (arrayOfExpression1 = expressions).length, b1 = 0; b1 < j; ) {
          Expression e = arrayOfExpression1[b1];
          if (e instanceof Annotation) {
            Annotation a = findAnnotation((Annotation)e, annotationBinding);
            if (a != null)
              return a; 
          } 
          b1++;
        } 
      } 
      b++;
    } 
    return null;
  }
}
